@@ -324,7 +324,7 @@ class MumbleCtlIce_118(MumbleCtlBase):
     @protectDjangoErrPage
     def setRegistration(self, srvid, mumbleid, name, email, password):
         import Murmur
-        user = Murmur.Player()
+        user = Murmur.RegisteredPlayer()
         user.playerid = mumbleid
         user.name     = name.encode( "UTF-8" )
         user.email    = email.encode( "UTF-8" )
@@ -430,6 +430,69 @@ class MumbleCtlIce_118(MumbleCtlBase):
     def getUptime(self, srvid):
         return None
 
+    @protectDjangoErrPage
+    def getBans( self, srvid ):
+        return self._getIceServerObject(srvid).getBans()
+
+    @protectDjangoErrPage
+    def getLog( self, srvid, first=0, last=100 ):
+        return self._getIceServerObject(srvid).getLog( first, last )
+
+    @protectDjangoErrPage
+    def addChannel( self, srvid, name, parentid ):
+        return self._getIceServerObject(srvid).addChannel( name.encode( "UTF-8" ), parentid )
+
+    @protectDjangoErrPage
+    def removeChannel( self, srvid, channelid ):
+        return self._getIceServerObject(srvid).removeChannel( channelid )
+
+    @protectDjangoErrPage
+    def renameChannel( self, srvid, channelid, name, description ):
+        srv = self._getIceServerObject(srvid)
+        state = srv.getChannelState(channelid)
+        state.name = name.encode("UTF-8")
+        srv.setChannelState(state)
+
+    @protectDjangoErrPage
+    def moveChannel(self, srvid, channelid, parentid):
+        srv = self._getIceServerObject(srvid)
+        state = srv.getChannelState(channelid)
+        state.parent = parentid
+        srv.setChannelState(state)
+
+    @protectDjangoErrPage
+    def moveUser(self, srvid, sessionid, channelid):
+        srv = self._getIceServerObject(srvid)
+        state = srv.getState(sessionid)
+        state.channel = channelid
+        srv.setState(state)
+
+    @protectDjangoErrPage
+    def muteUser(self, srvid, sessionid, mute=True):
+        srv = self._getIceServerObject(srvid)
+        state = srv.getState(sessionid)
+        state.mute = mute
+        srv.setState(state)
+
+    @protectDjangoErrPage
+    def deafenUser(self, srvid, sessionid, deaf=True):
+        srv = self._getIceServerObject(srvid)
+        state = srv.getState(sessionid)
+        state.deaf = deaf
+        srv.setState(state)
+
+    @protectDjangoErrPage
+    def kickUser(self, srvid, userid, reason=""):
+        return self._getIceServerObject(srvid).kickPlayer( userid, reason.encode("UTF-8") )
+
+    @protectDjangoErrPage
+    def sendMessage(self, srvid, sessionid, message):
+        return self._getIceServerObject(srvid).sendMessage( sessionid, message.encode( "UTF-8" ) )
+
+    @protectDjangoErrPage
+    def sendMessageChannel(self, srvid, channelid, tree, message):
+        return self._getIceServerObject(srvid).sendMessageChannel( channelid, tree, message.encode( "UTF-8" ) )
+
 
 class MumbleCtlIce_120(MumbleCtlIce_118):
     @protectDjangoErrPage
@@ -570,9 +633,17 @@ class MumbleCtlIce_120(MumbleCtlIce_118):
     def kickUser(self, srvid, userid, reason=""):
         return self._getIceServerObject(srvid).kickUser( userid, reason.encode("UTF-8") )
 
+    @protectDjangoErrPage
+    def renameChannel( self, srvid, channelid, name, description ):
+        srv = self._getIceServerObject(srvid)
+        state = srv.getChannelState(channelid)
+        state.name = name.encode("UTF-8")
+        state.description = description.encode("UTF-8")
+        srv.setChannelState(state)
+
     @protectDjangoErrPage
     def getUptime(self, srvid):
-        return self._getIceServerObject(srvid).getUptime()
+        return None
 
 
 class MumbleCtlIce_122(MumbleCtlIce_120):
@@ -587,6 +658,10 @@ class MumbleCtlIce_122(MumbleCtlIce_120):
         buf.seek(0)
         self._getIceServerObject(srvid).setTexture(mumbleid, buf.read())
 
+    @protectDjangoErrPage
+    def getUptime(self, srvid):
+        return None #self._getIceServerObject(srvid).getUptime()
+
 
 class MumbleCtlIce_123(MumbleCtlIce_120):
 
