@@ -15,7 +15,9 @@
  *  GNU General Public License for more details.
 """
 
-import Ice, re
+import re
+import datetime
+from time      import time
 from sys       import stderr
 from urllib    import urlopen
 from StringIO  import StringIO
@@ -213,11 +215,16 @@ class Mumble( models.Model ):
     textlen = mk_config_property( "textmessagelength",      ugettext_noop("Maximum length of text messages") )
     usersperchannel = mk_config_property( "usersperchannel",ugettext_noop("Users per channel"),  get_coerce=int )
 
-    obfsc   = mk_config_bool_property( "obfuscate",         ugettext_noop("IP Obfuscation") )
     certreq = mk_config_bool_property( "certrequired",      ugettext_noop("Require Certificate") )
     html    = mk_config_bool_property( "allowhtml",         ugettext_noop("Allow HTML to be used in messages") )
     bonjour = mk_config_bool_property( "bonjour",           ugettext_noop("Publish this server via Bonjour") )
     autoboot= mk_config_bool_property( "boot",              ugettext_noop("Boot Server when Murmur starts") )
+    rememchn= mk_config_bool_property( "rememberchannel",   ugettext_noop("Remember last channel") )
+
+    def get_absolute_url( self ):
+        from views import show
+        from django.core.urlresolvers import reverse
+        return reverse( show, kwargs={ 'server': self.id } )
 
     def getBooted( self ):
         if self.id is not None:
@@ -253,12 +260,12 @@ class Mumble( models.Model ):
             return u'Murmur "%s" (NOT YET CREATED)' % self.name
         return u'Murmur "%s" (%d)' % ( self.name, self.srvid )
 
-    def save( self, dontConfigureMurmur=False ):
+    def save( self, dontConfigureMurmur=False, *args, **kwargs ):
         """ Save the options configured in this model instance not only to Django's database,
             but to Murmur as well.
         """
         if dontConfigureMurmur:
-            return models.Model.save( self )
+            return models.Model.save( self, *args, **kwargs )
 
         if self.id is None:
             self.srvid = self.ctl.newServer()
@@ -280,7 +287,7 @@ class Mumble( models.Model ):
         else:
             self.ctl.setConf( self.srvid, 'registerhostname', '' )
 
-        return models.Model.save( self )
+        return models.Model.save( self, *args, **kwargs )
 
 
     def __init__( self, *args, **kwargs ):
@@ -293,9 +300,16 @@ class Mumble( models.Model ):
     users_online = property( lambda self: len(self.ctl.getPlayers(self.srvid)),  doc="Number of online users." )
     channel_cnt  = property( lambda self: len(self.ctl.getChannels(self.srvid)), doc="Number of channels." )
     is_public    = property( lambda self: not self.passwd,
-            doc="False if a password is needed to join this server." )
+                             doc="False if a password is needed to join this server." )
     uptime       = property( lambda self: self.ctl.getUptime(self.srvid),
-            doc="Number of seconds this instance has been running." )
+                             doc="Number of seconds this instance has been running." )
+
+    @property
+    def upsince(self):
+        """Datetime since when the server is running."""
+        if self.uptime is not None:
+            return datetime.datetime.fromtimestamp( float( time() - self.uptime ) )
+        return None
 
     is_server  = True
     is_channel = False
@@ -303,6 +317,9 @@ class Mumble( models.Model ):
 
     ctl = property( lambda self: self.server.ctl )
 
+    def getBans( self ):
+        return self.ctl.getBans( self.srvid )
+
     def getConf( self, field ):
         return self.ctl.getConf( self.srvid, field )
 
@@ -550,9 +567,59 @@ class Mumble( models.Model ):
     def kickUser( self, sessionid, reason="" ):
         return self.ctl.kickUser( self.srvid, sessionid, reason )
 
-    def banUser( self, sessionid, reason="" ):
-        return self.ctl.addBanForSession( self.srvid, sessionid, reason=reason )
+    def banUser( self, sessionid, reason="", duration=0 ):
+        return self.ctl.addBanForSession( self.srvid, sessionid, reason=reason, duration=duration )
 
+    def moveUser( self, sessionid, channelid ):
+        return self.ctl.moveUser( self.srvid, sessionid, channelid )
+
+    def muteUser( self, sessionid, mute=True ):
+        return self.ctl.muteUser( self.srvid, sessionid, mute )
+
+    def deafenUser( self, sessionid, deafen=True ):
+        return self.ctl.deafenUser( self.srvid, sessionid, deafen )
+
+    def sendMessage( self, sessionid, message ):
+        return self.ctl.sendMessage( self.srvid, sessionid, message )
+
+    def sendMessageChannel( self, channelid, tree, message ):
+        return self.ctl.sendMessageChannel( self.srvid, channelid, tree, message )
+
+    def hasUserTexture( self, userid ):
+        """ Check if this user has a texture set. """
+        if userid == -1:
+            return False
+        try:
+            self.getUserTexture( userid )
+        except ValueError:
+            return False
+        else:
+            return True
+
+    def getUserTexture( self, userid ):
+        return self.ctl.getTexture( self.srvid, userid )
+
+    def setUserTexture( self, userid, image ):
+        return self.ctl.setTexture( self.srvid, userid, image )
+
+    def addChannel( self, name, parentid ):
+        return self.ctl.addChannel( self.srvid, name, parentid )
+
+    def removeChannel( self, channelid ):
+        return self.ctl.removeChannel( self.srvid, channelid )
+
+    def renameChannel( self, channelid, name, description ):
+        return self.ctl.renameChannel( self.srvid, channelid, name, description )
+
+    def moveChannel( self, channelid, parentid ):
+        return self.ctl.moveChannel( self.srvid, channelid, parentid )
+
+    def getLog( self, first=0, last=100, filter="" ):
+        """ Return log entries from ``first`` to ``last`` that contain ``filter`` (if any). """
+        logentries = self.ctl.getLog( self.srvid, first, last )
+        if not filter:
+            return logentries
+        return [ logentry for logentry in logentries if filter in logentry.txt ]
 
 
 def mk_registration_property( field, doc="" ):
@@ -610,10 +677,10 @@ class MumbleUser( models.Model ):
             'du':  self.owner
             }
 
-    def save( self, dontConfigureMurmur=False ):
+    def save( self, dontConfigureMurmur=False, *args, **kwargs ):
         """ Save the settings in this model to Murmur. """
         if dontConfigureMurmur:
-            return models.Model.save( self )
+            return models.Model.save( self, *args, **kwargs )
 
         ctl = self.server.ctl
 
@@ -625,9 +692,9 @@ class MumbleUser( models.Model ):
         if self.id is None:
             # This is a new user record, so Murmur doesn't know about it yet
             if len( ctl.getRegisteredPlayers( self.server.srvid, self.name ) ) > 0:
-                raise ValueError( _( "Another player already registered that name." ) )
+                raise ValueError( "Another player already registered that name." )
             if not self.password:
-                raise ValueError( _( "Cannot register player without a password!" ) )
+                raise ValueError( "Cannot register player without a password!" )
 
             self.mumbleid = ctl.registerPlayer( self.server.srvid, self.name, email, self.password )
 
@@ -645,10 +712,10 @@ class MumbleUser( models.Model ):
         self.password = ''
 
         # If enabled (and possible), set Gravatar as default Avatar
-        if settings.USE_GRAVATAR and self.gravatar:
+        if not self.hasTexture and settings.USE_GRAVATAR and self.gravatar:
             self.setTextureFromUrl( self.gravatar )
 
-        return models.Model.save( self )
+        return models.Model.save( self, *args, **kwargs )
 
     def __init__( self, *args, **kwargs ):
         models.Model.__init__( self, *args, **kwargs )
@@ -688,29 +755,27 @@ class MumbleUser( models.Model ):
     # Texture handlers
     def getTexture( self ):
         """ Get the user texture as a PIL Image. """
-        return self.server.ctl.getTexture(self.server.srvid, self.mumbleid)
+        return self.server.getUserTexture( self.mumbleid )
 
     def setTexture( self, image ):
         """ Install the given image as the user's texture. """
-        self.server.ctl.setTexture(self.server.srvid, self.mumbleid, image)
+        self.server.setUserTexture( self.mumbleid, image )
 
     def setTextureFromUrl( self, url, transparency=None ):
         """ Retrieve the image at the given URL and set it as my texture. """
         img = Image.open( StringIO( urlopen( url ).read() ) )
         self.setTexture( img )
 
+    def setTextureFromGravatar( self ):
+        return self.setTextureFromUrl( self.gravatar )
+
     texture = property( getTexture, setTexture,
         doc="Get the texture as a PIL Image or set the Image as the texture."
         )
 
     def hasTexture( self ):
         """ Check if this user has a texture set. """
-        try:
-            self.getTexture()
-        except ValueError:
-            return False
-        else:
-            return True
+        return self.server.hasUserTexture( self.mumbleid )
 
     def gravatarUrl( self, size=80 ):
         """ Get a Gravatar URL for my owner's email adress (if any), or using the User's cert hash.
