@@ -15,7 +15,11 @@
  *  GNU General Public License for more details.
 """
 
-import simplejson
+try:
+    import simplejson
+except ImportError:
+    import json as simplejson
+
 from StringIO     import StringIO
 from PIL          import Image
 
@@ -27,11 +31,19 @@ from django.contrib.auth.decorators import login_required
 from django.contrib.auth.models     import User
 from django.contrib.auth            import views as auth_views
 from django.core.urlresolvers       import reverse
+from django.views.decorators.csrf   import csrf_exempt
 
 from models import Mumble, MumbleUser
 from forms  import MumbleForm, MumbleUserForm, MumbleUserPasswordForm
 from forms  import MumbleUserLinkForm, MumbleTextureForm, MumbleKickForm
 
+from djextdirect.provider import Provider
+from djextdirect.views    import login, logout
+
+EXT_DIRECT_PROVIDER = Provider()
+
+EXT_DIRECT_PROVIDER._register_method( "Accounts", login  )
+EXT_DIRECT_PROVIDER._register_method( "Accounts", logout )
 
 def redir( request ):
     """ Redirect to the servers list. """
@@ -82,6 +94,39 @@ def mumbles( request, mobile=False ):
         context_instance = RequestContext(request)
         )
 
+@EXT_DIRECT_PROVIDER.register_method( "MumbleList" )
+def servers( request ):
+    mms = Mumble.objects.all().order_by( "name" )
+    return [{ 'id': mm.id, 'name': mm.name, 'booted': mm.booted } for mm in mms]
+
+@EXT_DIRECT_PROVIDER.register_method( "MumbleList" )
+def serverinfo( request, server ):
+    srv = Mumble.objects.get( id=int(server) )
+    if srv.booted:
+        return {
+            'id':            srv.id,
+            'name':          srv.name,
+            'booted':        True,
+            'motd':          srv.motd,
+            'connecturl':    srv.connecturl,
+            'prettyversion': srv.prettyversion,
+            'url':           srv.url,
+            'users_regged':  srv.users_regged,
+            'users_online':  srv.users_online,
+            'channel_cnt':   srv.channel_cnt,
+            'uptime':        srv.uptime,
+            'upsince':       unicode(srv.upsince),
+            'minurl':        reverse( mobile_show, args=(server,) ),
+            'detailsurl':    reverse( show,        args=(server,) ),
+            }
+    else:
+        return{
+            'id':            srv.id,
+            'name':          srv.name,
+            'booted':        False,
+            'minurl':        reverse( mobile_show, args=(server,) ),
+            'detailsurl':    reverse( show,        args=(server,) ),
+            }
 
 def show( request, server ):
     """ Display the channel list for the given Server ID.
@@ -99,116 +144,45 @@ def show( request, server ):
             }, context_instance = RequestContext(request) )
 
     isAdmin = srv.isUserAdmin( request.user )
-
-    # The tab to start on.
-    displayTab = 0
-
-    if isAdmin:
-        if request.method == 'POST' and "mode" in request.POST and request.POST['mode'] == 'admin':
-            adminform = MumbleForm( request.POST, instance=srv )
-            if adminform.is_valid():
-                adminform.save()
-                return HttpResponseRedirect( reverse( show, kwargs={ 'server': int(server), } ) )
-            else:
-                displayTab = 2
-        else:
-            adminform = MumbleForm( instance=srv )
-    else:
-        adminform = None
-
-    registered = False
-    user = None
-
     if request.user.is_authenticated():
+        try:
+            user = MumbleUser.objects.get( server=srv, owner=request.user )
+        except MumbleUser.DoesNotExist:
+            user = None
+    else:
+        user = None
+
+    from mumble.forms import EXT_FORMS_PROVIDER, MumbleUserPasswordForm, MumbleUserLinkForm, MumbleUserForm
+
+    regformname = None
+    if not user:
         # Unregistered users may or may not need a password to register.
         if settings.PROTECTED_MODE and srv.passwd:
-            unregged_user_form = MumbleUserPasswordForm
+            regformname = "MumbleUserPasswordForm"
+            EXT_FORMS_PROVIDER.register_form( MumbleUserPasswordForm )
         # Unregistered users may or may not want to link an existing account
         elif settings.ALLOW_ACCOUNT_LINKING:
-            unregged_user_form = MumbleUserLinkForm
-        else:
-            unregged_user_form = MumbleUserForm
-
-
-        if request.method == 'POST' and 'mode' in request.POST and request.POST['mode'] == 'reg':
-            try:
-                user    = MumbleUser.objects.get( server=srv, owner=request.user )
-            except MumbleUser.DoesNotExist:
-                regform = unregged_user_form( request.POST )
-                regform.server = srv
-                if regform.is_valid():
-                    model = regform.save( commit=False )
-                    model.owner  = request.user
-                    model.server = srv
-                    # If we're linking accounts, the change is local only.
-                    model.save( dontConfigureMurmur=( "linkacc" in regform.data ) )
-                    return HttpResponseRedirect( reverse( show, kwargs={ 'server': int(server), } ) )
-                else:
-                    displayTab = 1
-            else:
-                regform = MumbleUserForm( request.POST, instance=user )
-                regform.server = srv
-                if regform.is_valid():
-                    regform.save()
-                    return HttpResponseRedirect( reverse( show, kwargs={ 'server': int(server), } ) )
-                else:
-                    displayTab = 1
-        else:
-            try:
-                user  = MumbleUser.objects.get( server=srv, owner=request.user )
-            except MumbleUser.DoesNotExist:
-                regform = unregged_user_form()
-            else:
-                regform = MumbleUserForm( instance=user )
-                registered = True
-
-        if request.method == 'POST' and 'mode' in request.POST and request.POST['mode'] == 'texture' and registered:
-            textureform = MumbleTextureForm( request.POST, request.FILES )
-            if textureform.is_valid():
-                if 'usegravatar' in textureform.cleaned_data and textureform.cleaned_data['usegravatar'] and user.gravatar:
-                    user.setTextureFromUrl( user.gravatar )
-                elif 'texturefile' in request.FILES:
-                    user.setTexture( Image.open( request.FILES['texturefile'] ) )
-                return HttpResponseRedirect( reverse( show, kwargs={ 'server': int(server), } ) )
-        else:
-            textureform = MumbleTextureForm()
+            regformname = "MumbleUserLinkForm"
+            EXT_FORMS_PROVIDER.register_form( MumbleUserLinkForm )
+    if not regformname:
+        regformname = "MumbleUserForm"
+        EXT_FORMS_PROVIDER.register_form( MumbleUserForm )
 
+    try:
+        import qrencode
+    except ImportError:
+        qravail = False
     else:
-        regform = None
-        textureform = None
-
-    if isAdmin:
-        if request.method == 'POST' and 'mode' in request.POST:
-            if request.POST['mode'] == 'kick':
-                kickform = MumbleKickForm( request.POST )
-                if kickform.is_valid():
-                    if kickform.cleaned_data["ban"]:
-                        srv.banUser( kickform.cleaned_data['session'], kickform.cleaned_data['reason'] )
-                    srv.kickUser( kickform.cleaned_data['session'], kickform.cleaned_data['reason'] )
-
-    # ChannelTable is a somewhat misleading name, as it actually contains channels and players.
-    channelTable = []
-    for cid in srv.channels:
-        if cid != 0 and srv.channels[cid].show:
-            channelTable.append( srv.channels[cid] )
-    for pid in srv.players:
-        channelTable.append( srv.players[pid] )
-
-    show_url = reverse( show, kwargs={ 'server': srv.id } )
-    login_url = reverse( auth_views.login )
+        qravail = True
 
     return render_to_response( 'mumble/mumble.html', {
-            'login_url':    "%s?next=%s" % ( login_url, show_url ),
-            'DBaseObject':  srv,
-            'ChannelTable': channelTable,
-            'CurrentUserIsAdmin': isAdmin,
-            'AdminForm':    adminform,
-            'RegForm':      regform,
-            'TextureForm':  textureform,
-            'Registered':   registered,
-            'DisplayTab':   displayTab,
+            'MumbleServer': srv,
+            'ServerDict':   simplejson.dumps(serverinfo(request, server)),
+            'RegForm':      regformname,
             'MumbleActive': True,
             'MumbleAccount':user,
+            'QRAvailable':  qravail,
+            'IsAdmin':      isAdmin,
         }, context_instance = RequestContext(request) )
 
 def mobile_show( request, server ):
@@ -229,18 +203,34 @@ def mobile_show( request, server ):
             'MumbleAccount':user,
         }, context_instance = RequestContext(request) )
 
+def embed( request, server ):
+    """ Display the channel viewer for the given Server ID. """
+
+    srv = get_object_or_404( Mumble, id=server )
+
+    return render_to_response( 'mumble/embed.html', {
+            'MumbleServer': srv,
+            'MumbleActive': True,
+        }, context_instance = RequestContext(request) )
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def hasTexture( request, server, userid ):
+    srv = get_object_or_404( Mumble,     id=int(server) )
+    if srv.hasUserTexture(int(userid)):
+        return {
+            'has': True,
+            'url': reverse( showTexture, kwargs={ 'server': server, 'userid': userid } )
+            }
+    else:
+        return { 'has': False, 'url': None }
 
 def showTexture( request, server, userid ):
-    """ Pack the given user's texture into an HttpResponse.
-
-        If userid is none, use the currently logged in User.
-    """
+    """ Pack the given user's texture into an HttpResponse. """
 
     srv  = get_object_or_404( Mumble,     id=int(server) )
-    user = get_object_or_404( MumbleUser, server=srv, id=int(userid) )
 
     try:
-        img  = user.getTexture()
+        img  = srv.getUserTexture(int(userid))
     except ValueError:
         raise Http404()
     else:
@@ -248,9 +238,132 @@ def showTexture( request, server, userid ):
         img.save( buf, "PNG" )
         return HttpResponse( buf.getvalue(), "image/png" )
 
+def qrcode( request, server ):
+    """ Show a QR Coce image that links to the server's embedded page. """
 
-@login_required
-def users( request, server ):
+    try:
+        import qrencode
+    except ImportError:
+        return HttpResponse( "oh noez" )
+    else:
+        from django.contrib.sites.models import Site
+        version, size, img = qrencode.encode_scaled( "%s://%s%s" % (
+            { False: "http", True: "https" }[request.is_secure()],
+            Site.objects.get_current().domain,
+            reverse( embed, kwargs={ 'server': server } )
+            ), 82 )
+        buf = StringIO()
+        img.save( buf, "PNG" )
+        return HttpResponse( buf.getvalue(), "image/png" )
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def get_admin( request, server ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( 'Access denied' )
+    adminform = MumbleForm( request.POST, instance=srv )
+    data = {}
+    for fld in adminform.fields:
+        data[fld] = getattr( srv, fld )
+    return { 'data': data, 'success': True }
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def log( request, server, start, limit, filter ):
+    """ Retrieve log messages. """
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( "Access denied" )
+    return { 'data': [
+            { 'timestamp': ent.timestamp, 'txt': ent.txt }
+            for ent in srv.getLog( start, (start + limit), filter )
+        ], 'success': True }
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def bans( request, server ):
+    """ Retrieve log messages. """
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( "Access denied" )
+    return { 'data': [
+            { 'start': ent.start, 'address': ent.address, 'bits': ent.bits,
+              'duration': ent.duration, 'reason': ent.reason }
+            for ent in srv.getBans()
+        ], 'success': True }
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def moveUser( request, server, sessionid, channelid ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( 'Access denied' )
+    srv.moveUser( sessionid, channelid )
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def moveChannel( request, server, channelid, parentid ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( 'Access denied' )
+    srv.moveChannel( channelid, parentid )
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def kickUser( request, server, sessionid, reason, ban, duration ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( 'Access denied' )
+    if ban:
+        srv.banUser( sessionid, reason, duration )
+    srv.kickUser( sessionid, reason )
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def muteUser( request, server, sessionid, mute ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( 'Access denied' )
+    srv.muteUser(sessionid, mute)
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def deafenUser( request, server, sessionid, deaf ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( 'Access denied' )
+    srv.deafenUser(sessionid, deaf)
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def addChannel( request, server, name, parentid ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( 'Access denied' )
+    srv.addChannel(name, parentid)
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def removeChannel( request, server, channelid ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( 'Access denied' )
+    srv.removeChannel(channelid)
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def renameChannel( request, server, channelid, name, description ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( 'Access denied' )
+    srv.renameChannel(channelid, name, description)
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def sendMessage( request, server, sessionid, message ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( 'Access denied' )
+    srv.sendMessage(sessionid, message)
+
+@EXT_DIRECT_PROVIDER.register_method( "Mumble" )
+def sendMessageChannel( request, server, channelid, tree, message ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    if not srv.isUserAdmin( request.user ):
+        raise Exception( 'Access denied' )
+    srv.sendMessageChannel(channelid, tree, message)
+
+@EXT_DIRECT_PROVIDER.register_method( "MumbleUserAdmin" )
+def users( request, server, resync=False ):
     """ Create a list of MumbleUsers for a given server serialized as a JSON object.
 
         If the request has a "data" field, evaluate that and update the user records.
@@ -258,34 +371,11 @@ def users( request, server ):
 
     srv = get_object_or_404( Mumble, id=int(server) )
 
-    if "resync" in request.POST and request.POST['resync'] == "true":
+    if resync:
         srv.readUsersFromMurmur()
 
     if not srv.isUserAdmin( request.user ):
-        return HttpResponse(
-            simplejson.dumps({ 'success': False, 'objects': [], 'errormsg': 'Access denied' }),
-            mimetype='text/javascript'
-            )
-
-    if request.method == 'POST':
-        data = simplejson.loads( request.POST['data'] )
-        for record in data:
-            if record['id'] == -1:
-                if record['delete']:
-                    continue
-                mu = MumbleUser( server=srv )
-            else:
-                mu = MumbleUser.objects.get( id=record['id'] )
-                if record['delete']:
-                    mu.delete()
-                    continue
-
-            mu.name     = record['name']
-            mu.password = record['password']
-            if record['owner']:
-                mu.owner = User.objects.get( id=int(record['owner']) )
-            mu.save()
-            mu.aclAdmin = record['admin']
+        raise Exception( 'Access denied' )
 
     users = []
     for mu in srv.mumbleuser_set.all():
@@ -301,13 +391,9 @@ def users( request, server ):
             'admin':    mu.aclAdmin,
             } )
 
-    return HttpResponse(
-        simplejson.dumps( { 'success': True, 'objects': users } ),
-        mimetype='text/javascript'
-        )
+    return users
 
-
-@login_required
+@EXT_DIRECT_PROVIDER.register_method( "MumbleUserAdmin" )
 def djangousers( request ):
     """ Return a list of all Django users' names and IDs. """
 
@@ -318,11 +404,47 @@ def djangousers( request ):
             'uname': unicode( du ),
             } )
 
-    return HttpResponse(
-        simplejson.dumps( { 'success': True, 'objects': users } ),
-        mimetype='text/javascript'
-        )
+    return users
 
+@EXT_DIRECT_PROVIDER.register_method( "MumbleUserAdmin" )
+def update( request, server, data ):
+    srv = get_object_or_404( Mumble, id=int(server) )
+    for record in data:
+        if record['id'] == -1:
+            if record['delete']:
+                continue
+            mu = MumbleUser( server=srv )
+        else:
+            mu = MumbleUser.objects.get( id=record['id'] )
+            if record['delete']:
+                mu.delete()
+                continue
+
+        mu.name     = record['name']
+        mu.password = record['password']
+        if record['owner']:
+            mu.owner = User.objects.get( id=int(record['owner']) )
+        mu.save()
+        mu.aclAdmin = record['admin']
+    return { 'success': True }
+
+@login_required
+@csrf_exempt
+def update_avatar( request, userid ):
+    try:
+        user = MumbleUser.objects.get( id=userid )
+    except MumbleUser.DoesNotExist:
+        return HttpResponse( "false", mimetype="text/html" )
+
+    textureform = MumbleTextureForm( request.POST, request.FILES )
+    if textureform.is_valid():
+        if textureform.cleaned_data['usegravatar'] and user.gravatar:
+            user.setTextureFromUrl( user.gravatar )
+        else:
+            user.setTexture( Image.open( textureform.cleaned_data['texturefile'] ) )
+        return HttpResponse( "true", mimetype="text/html" )
+
+    return HttpResponse( "false", mimetype="text/html" )
 
 def mmng_tree( request, server ):
     """ Return a JSON representation of the channel tree suitable for
