@@ -17,12 +17,17 @@
 
 def installed_apps(request):
     from django.conf import settings
-    return { 'ROSETTA_INSTALLED': "rosetta" in settings.INSTALLED_APPS }
+    return { 'INSTALLED_APPS': settings.INSTALLED_APPS }
 
 def mumble_version(request):
     from mumble import version_str
     return { 'CURRENTVERSION': version_str }
 
+def mumble_media_prefix(request):
+    from django.conf import settings
+    return { 'MUMBLE_MEDIA_PREFIX': settings.MUMBLE_MEDIA_PREFIX,
+             'MUMBLE_DJANGO_URL':   settings.MUMBLE_DJANGO_URL}
+
 def theme_url(request):
     from django.conf import settings
     if settings.THEME:
